@@ -77,6 +77,7 @@ struct _GGduVolume
   GFile *activation_root;
   gchar *name;
   gchar *device_file;
+  dev_t dev;
   gchar *uuid;
   gboolean can_mount;
   gboolean should_automount;
@@ -102,6 +103,18 @@ static void gdu_cleartext_volume_job_changed (GduPresentable *presentable,
 
 static void mount_with_mount_operation (MountOpData *data);
 
+static gboolean
+g_gdu_drive_device_is_flash(GduDevice *drive_device)
+{
+  const gchar *drive_media;
+  
+  if (drive_device == NULL)
+    return FALSE;
+  
+  drive_media = gdu_device_drive_get_media (drive_device);
+  return g_str_has_prefix (drive_media, "flash");
+}
+
 static void
 g_gdu_volume_finalize (GObject *object)
 {
@@ -172,13 +185,14 @@ static gboolean
 update_volume (GGduVolume *volume)
 {
   GduDevice *device;
-  GduPool *pool;
+  GduPool *pool = NULL;
   time_t now;
   gboolean changed;
   gboolean old_can_mount;
   gboolean old_should_automount;
   gchar *old_name;
   gchar *old_device_file;
+  dev_t old_dev;
   GIcon *old_icon;
   gboolean keep_cleartext_volume;
 
@@ -187,6 +201,7 @@ update_volume (GGduVolume *volume)
   old_should_automount = volume->should_automount;
   old_name = g_strdup (volume->name);
   old_device_file = g_strdup (volume->device_file);
+  old_dev = volume->dev;
   old_icon = volume->icon != NULL ? g_object_ref (volume->icon) : NULL;
 
   /* ---------------------------------------------------------------------------------------------------- */
@@ -194,11 +209,14 @@ update_volume (GGduVolume *volume)
   /* if the volume is a fstab mount point, get the data from there */
   if (volume->unix_mount_point != NULL)
     {
+      struct stat_buf;
+
       volume->can_mount = TRUE;
       volume->should_automount = FALSE;
 
       g_free (volume->device_file);
       volume->device_file = g_strdup (g_unix_mount_point_get_device_path (volume->unix_mount_point));
+      volume->dev = 0;
 
       if (volume->icon != NULL)
         g_object_unref (volume->icon);
@@ -217,10 +235,11 @@ update_volume (GGduVolume *volume)
 
   /* in with the new */
   device = gdu_presentable_get_device (GDU_PRESENTABLE (volume->gdu_volume));
-  pool = gdu_device_get_pool (device);
+  if (device != NULL)
+    pool = gdu_device_get_pool (device);
 
   keep_cleartext_volume = FALSE;
-  if (gdu_device_is_luks (device))
+  if (device != NULL && gdu_device_is_luks (device))
     {
       const gchar *holder_objpath;
 
@@ -287,13 +306,23 @@ update_volume (GGduVolume *volume)
       volume->name = gdu_presentable_get_name (GDU_PRESENTABLE (volume->cleartext_gdu_volume));
 
       g_free (volume->device_file);
-      volume->device_file = g_strdup (gdu_device_get_device_file (luks_cleartext_volume_device));
+      if (luks_cleartext_volume_device != NULL)
+        {
+          volume->device_file = g_strdup (gdu_device_get_device_file (luks_cleartext_volume_device));
+          volume->dev = gdu_device_get_dev (luks_cleartext_volume_device);
+        }
+      else
+        {
+          volume->device_file = NULL;
+          volume->dev = 0;
+        }
 
       volume->can_mount = TRUE;
 
       volume->should_automount = FALSE;
 
-      g_object_unref (luks_cleartext_volume_device);
+      if (luks_cleartext_volume_device != NULL)
+        g_object_unref (luks_cleartext_volume_device);
     }
   else
     {
@@ -321,17 +350,70 @@ update_volume (GGduVolume *volume)
         }
 
       g_free (volume->device_file);
-      volume->device_file = g_strdup (gdu_device_get_device_file (device));
+      if (device != NULL)
+        {
+          volume->device_file = g_strdup (gdu_device_get_device_file (device));
+          volume->dev = gdu_device_get_dev (device);
+        }
+      else
+        {
+          volume->device_file = NULL;
+          volume->dev = 0;
+        }
 
       volume->can_mount = TRUE;
 
-      /* If a volume (partition) appear _much later_ than when media was insertion it
-       * can only be because the media was repartitioned. We don't want to automount
-       * such volumes.
+      /* Only automount filesystems from drives of known types/interconnects:
+       *
+       *  - USB
+       *  - Firewire
+       *  - sdio
+       *  - optical discs
+       *
+       * The mantra here is "be careful" - we really don't want to
+       * automount fs'es from all devices in a SAN etc - We REALLY
+       * need to be CAREFUL here.
+       *
+       * Sidebar: Actually, a surprisingly large number of admins like
+       *          to log into GNOME as root (thus having all polkit
+       *          authorizations) and if weren't careful we'd
+       *          automount all mountable devices from the box. See
+       *          the enterprise distro bug trackers for details.
        */
-      volume->should_automount = TRUE;
+      volume->should_automount = FALSE;
       if (volume->drive != NULL)
         {
+          GduPresentable *drive_presentable;
+          drive_presentable = g_gdu_drive_get_presentable (volume->drive);
+          if (drive_presentable != NULL)
+            {
+              GduDevice *drive_device;
+              drive_device = gdu_presentable_get_device (drive_presentable);
+              if (drive_device != NULL)
+                {
+                  if (gdu_device_is_drive (drive_device))
+                    {
+                      const gchar *connection_interface;
+
+                      connection_interface = gdu_device_drive_get_connection_interface (drive_device);
+
+                      if (g_strcmp0 (connection_interface, "usb") == 0 ||
+                          g_strcmp0 (connection_interface, "firewire") == 0 ||
+                          g_strcmp0 (connection_interface, "sdio") == 0 ||
+                          g_gdu_drive_device_is_flash (drive_device) ||
+                          gdu_device_is_optical_disc (drive_device))
+                        {
+                          volume->should_automount = TRUE;
+                        }
+                    }
+                  g_object_unref (drive_device);
+                }
+            }
+
+          /* If a volume (partition) appear _much later_ than when media was inserted it
+           * can only be because the media was repartitioned. We don't want to automount
+           * such volumes.
+           */
           now = time (NULL);
           if (now - g_gdu_drive_get_time_of_last_media_insertion (volume->drive) > 5)
             volume->should_automount = FALSE;
@@ -341,14 +423,16 @@ update_volume (GGduVolume *volume)
        * nopolicy is only FALSE for "physical" devices - e.g. only "physical" devices will
        * be set to be automounted.
        */
-      if (gdu_device_get_presentation_nopolicy (device))
+      if (device != NULL && gdu_device_get_presentation_nopolicy (device))
         volume->should_automount = FALSE;
 
       g_free (activation_uri);
     }
 
-  g_object_unref (pool);
-  g_object_unref (device);
+  if (pool != NULL)
+    g_object_unref (pool);
+  if (device != NULL)
+    g_object_unref (device);
 
   /* ---------------------------------------------------------------------------------------------------- */
 
@@ -359,6 +443,7 @@ update_volume (GGduVolume *volume)
               (old_should_automount == volume->should_automount) &&
               (g_strcmp0 (old_name, volume->name) == 0) &&
               (g_strcmp0 (old_device_file, volume->device_file) == 0) &&
+              (old_dev == volume->dev) &&
               g_icon_equal (old_icon, volume->icon)
               );
 
@@ -978,7 +1063,21 @@ mount_with_mount_operation (MountOpData *data)
 
   device = gdu_presentable_get_device (GDU_PRESENTABLE (data->volume->gdu_volume));
 
-  toplevel = gdu_presentable_get_toplevel (GDU_PRESENTABLE (data->volume->gdu_volume));
+  toplevel = gdu_presentable_get_enclosing_presentable (GDU_PRESENTABLE (data->volume->gdu_volume));
+  /* handle logical partitions enclosed by an extented partition */
+  if (GDU_IS_VOLUME (toplevel))
+    {
+      GduPresentable *temp;
+      temp = toplevel;
+      toplevel = gdu_presentable_get_enclosing_presentable (toplevel);
+      g_object_unref (temp);
+      if (!GDU_IS_DRIVE (toplevel))
+        {
+          g_object_unref (toplevel);
+          toplevel = NULL;
+        }
+    }
+
   if (toplevel != NULL)
     drive_name = gdu_presentable_get_name (toplevel);
 
@@ -1082,6 +1181,20 @@ g_gdu_volume_mount (GVolume             *_volume,
     }
 
   device = gdu_presentable_get_device (GDU_PRESENTABLE (volume->gdu_volume));
+
+  if (device == NULL)
+    {
+      simple = g_simple_async_result_new_error (G_OBJECT (volume),
+                                                callback,
+                                                user_data,
+                                                G_IO_ERROR,
+                                                G_IO_ERROR_FAILED,
+                                                "Underlying device missing");
+      g_simple_async_result_complete (simple);
+      g_object_unref (simple);
+      goto out;
+    }
+
   pool = gdu_device_get_pool (device);
 
   /* Makes no sense to mount
@@ -1129,11 +1242,14 @@ g_gdu_volume_mount (GVolume             *_volume,
 
       luks_cleartext_volume_device = gdu_presentable_get_device (GDU_PRESENTABLE (volume->cleartext_gdu_volume));
 
-      object_path_of_cleartext_device = gdu_device_get_object_path (luks_cleartext_volume_device);
+      if (luks_cleartext_volume_device != NULL)
+        {
+          object_path_of_cleartext_device = gdu_device_get_object_path (luks_cleartext_volume_device);
 
-      mount_cleartext_device (data, object_path_of_cleartext_device);
+          mount_cleartext_device (data, object_path_of_cleartext_device);
 
-      g_object_unref (luks_cleartext_volume_device);
+          g_object_unref (luks_cleartext_volume_device);
+        }
       goto out;
     }
 
@@ -1551,17 +1667,20 @@ g_gdu_volume_get_identifier (GVolume     *_volume,
     {
       device = gdu_presentable_get_device (GDU_PRESENTABLE (volume->gdu_volume));
 
-      label = gdu_device_id_get_label (device);
-      uuid = gdu_device_id_get_uuid (device);
+      if (device != NULL)
+        {
+          label = gdu_device_id_get_label (device);
+          uuid = gdu_device_id_get_uuid (device);
 
-      g_object_unref (device);
+          g_object_unref (device);
 
-      if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE) == 0)
-        id = g_strdup (volume->device_file);
-      else if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_LABEL) == 0)
-        id = strlen (label) > 0 ? g_strdup (label) : NULL;
-      else if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_UUID) == 0)
-        id = strlen (uuid) > 0 ? g_strdup (uuid) : NULL;
+          if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE) == 0)
+            id = g_strdup (volume->device_file);
+          else if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_LABEL) == 0)
+            id = strlen (label) > 0 ? g_strdup (label) : NULL;
+          else if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_UUID) == 0)
+            id = strlen (uuid) > 0 ? g_strdup (uuid) : NULL;
+        }
     }
 
   return id;
@@ -1577,19 +1696,24 @@ g_gdu_volume_enumerate_identifiers (GVolume *_volume)
   const gchar *uuid;
 
   p = g_ptr_array_new ();
+  label = NULL;
 
   if (volume->gdu_volume != NULL)
     {
       device = gdu_presentable_get_device (GDU_PRESENTABLE (volume->gdu_volume));
-      label = gdu_device_id_get_label (device);
-      uuid = gdu_device_id_get_uuid (device);
-      g_object_unref (device);
 
-      g_ptr_array_add (p, g_strdup (G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE));
-      if (strlen (label) > 0)
-        g_ptr_array_add (p, g_strdup (G_VOLUME_IDENTIFIER_KIND_LABEL));
-      if (strlen (uuid) > 0)
-        g_ptr_array_add (p, g_strdup (G_VOLUME_IDENTIFIER_KIND_UUID));
+      if (device != NULL)
+        {
+          label = gdu_device_id_get_label (device);
+          uuid = gdu_device_id_get_uuid (device);
+          g_object_unref (device);
+
+          g_ptr_array_add (p, g_strdup (G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE));
+          if (strlen (label) > 0)
+            g_ptr_array_add (p, g_strdup (G_VOLUME_IDENTIFIER_KIND_LABEL));
+          if (strlen (uuid) > 0)
+            g_ptr_array_add (p, g_strdup (G_VOLUME_IDENTIFIER_KIND_UUID));
+        }
     }
 
   g_ptr_array_add (p, NULL);
@@ -1626,6 +1750,28 @@ g_gdu_volume_volume_iface_init (GVolumeIface *iface)
   iface->get_activation_root = g_gdu_volume_get_activation_root;
 }
 
+gboolean
+g_gdu_volume_has_dev (GGduVolume   *volume,
+                      dev_t         dev)
+{
+  dev_t _dev;
+
+  _dev = volume->dev;
+
+  if (volume->cleartext_gdu_volume != NULL)
+    {
+      GduDevice *luks_cleartext_volume_device;
+      luks_cleartext_volume_device = gdu_presentable_get_device (GDU_PRESENTABLE (volume->cleartext_gdu_volume));
+      if (luks_cleartext_volume_device != NULL)
+        {
+          _dev = gdu_device_get_dev (luks_cleartext_volume_device);
+          g_object_unref (luks_cleartext_volume_device);
+        }
+    }
+
+  return _dev == dev;
+}
+
 gboolean
 g_gdu_volume_has_device_file (GGduVolume   *volume,
                               const gchar  *device_file)
@@ -1638,8 +1784,11 @@ g_gdu_volume_has_device_file (GGduVolume   *volume,
     {
       GduDevice *luks_cleartext_volume_device;
       luks_cleartext_volume_device = gdu_presentable_get_device (GDU_PRESENTABLE (volume->cleartext_gdu_volume));
-      _device_file = gdu_device_get_device_file (luks_cleartext_volume_device);
-      g_object_unref (luks_cleartext_volume_device);
+      if (luks_cleartext_volume_device != NULL)
+        {
+          _device_file = gdu_device_get_device_file (luks_cleartext_volume_device);
+          g_object_unref (luks_cleartext_volume_device);
+        }
     }
 
   return g_strcmp0 (_device_file, device_file) == 0;
@@ -1682,8 +1831,11 @@ g_gdu_volume_has_uuid (GGduVolume  *volume,
     {
       GduDevice *luks_cleartext_volume_device;
       luks_cleartext_volume_device = gdu_presentable_get_device (GDU_PRESENTABLE (volume->cleartext_gdu_volume));
-      _uuid = gdu_device_id_get_uuid (luks_cleartext_volume_device);
-      g_object_unref (luks_cleartext_volume_device);
+      if (luks_cleartext_volume_device != NULL)
+        {
+          _uuid = gdu_device_id_get_uuid (luks_cleartext_volume_device);
+          g_object_unref (luks_cleartext_volume_device);
+        }
     }
 
   return g_strcmp0 (_uuid, uuid) == 0;
@@ -1712,3 +1864,12 @@ g_gdu_volume_get_unix_mount_point (GGduVolume *volume)
 {
   return volume->unix_mount_point;
 }
+
+gboolean
+g_gdu_volume_has_presentable (GGduVolume       *volume,
+                              GduPresentable  *presentable)
+{
+  return volume->gdu_volume != NULL &&
+    g_strcmp0 (gdu_presentable_get_id (GDU_PRESENTABLE (volume->gdu_volume)),
+               gdu_presentable_get_id (presentable)) == 0;
+}
